package sun.security.util;

import java.util.Arrays;
import java.math.BigInteger;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;

public final class ObjectIdentifier implements Serializable
{
    private static final int MAXIMUM_OID_SIZE = 4096;
    private byte[] encoding;
    private transient volatile String stringForm;
    private static final long serialVersionUID = 8697030238860181294L;
    private Object components;
    private int componentLen;
    private transient boolean componentsCalculated;
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        if (this.encoding == null) {
            int[] array = (int[])(int[])this.components;
            if (this.componentLen > array.length) {
                this.componentLen = array.length;
            }
            checkOidSize(this.componentLen);
            this.init(array, this.componentLen);
        }
        else {
            checkOidSize(this.encoding.length);
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        if (!this.componentsCalculated) {
            int[] intArray = this.toIntArray();
            if (intArray != null) {
                this.components = intArray;
                this.componentLen = intArray.length;
            }
            else {
                this.components = ObjectIdentifier.HugeOidNotSupportedByOldJDK.theOne;
            }
            this.componentsCalculated = true;
        }
        objectOutputStream.defaultWriteObject();
    }
    
    public ObjectIdentifier(String stringForm) throws IOException {
        this.encoding = null;
        this.components = null;
        this.componentLen = -1;
        this.componentsCalculated = false;
        int ch = 46;
        int beginIndex = 0;
        int n = 0;
        byte[] array = new byte[stringForm.length()];
        int intValue = 0;
        int n2 = 0;
        while (true) {
            try {
                int index = stringForm.indexOf(ch, beginIndex);
                String s;
                int n3;
                if (index == -1) {
                    s = stringForm.substring(beginIndex);
                    n3 = stringForm.length() - beginIndex;
                }
                else {
                    s = stringForm.substring(beginIndex, index);
                    n3 = index - beginIndex;
                }
                if (n3 > 9) {
                    BigInteger add = new BigInteger(s);
                    if (n2 == 0) {
                        checkFirstComponent(add);
                        intValue = add.intValue();
                    }
                    else {
                        if (n2 == 1) {
                            checkSecondComponent(intValue, add);
                            add = add.add(BigInteger.valueOf((long)(40 * intValue)));
                        }
                        else {
                            checkOtherComponent(n2, add);
                        }
                        n += pack7Oid(add, array, n);
                    }
                }
                else {
                    int int1 = Integer.parseInt(s);
                    if (n2 == 0) {
                        checkFirstComponent(int1);
                        intValue = int1;
                    }
                    else {
                        if (n2 == 1) {
                            checkSecondComponent(intValue, int1);
                            int1 += 40 * intValue;
                        }
                        else {
                            checkOtherComponent(n2, int1);
                        }
                        n += pack7Oid(int1, array, n);
                    }
                }
                beginIndex = index + 1;
                ++n2;
                checkOidSize(n);
                continue;
                checkCount(n2);
                System.arraycopy(array, 0, this.encoding = new byte[n], 0, n);
                this.stringForm = stringForm;
                break;
            }
            catch (IOException ex) {
                throw ex;
            }
            catch (Exception cause) {
                throw new IOException("ObjectIdentifier() -- Invalid format: " + cause.toString(), cause);
            }
        }
    }
    
    public ObjectIdentifier(int[] array) throws IOException {
        this.encoding = null;
        this.components = null;
        this.componentLen = -1;
        this.componentsCalculated = false;
        checkCount(array.length);
        checkFirstComponent(array[0]);
        checkSecondComponent(array[0], array[1]);
        for (int i = 2; i < array.length; ++i) {
            checkOtherComponent(i, array[i]);
        }
        this.init(array, array.length);
    }
    
    public ObjectIdentifier(DerInputStream derInputStream) throws IOException {
        this.encoding = null;
        this.components = null;
        this.componentLen = -1;
        this.componentsCalculated = false;
        int i = (byte)derInputStream.getByte();
        if (i != 6) {
            throw new IOException("ObjectIdentifier() -- data isn't an object ID (tag = " + i + ")");
        }
        int definiteLength = derInputStream.getDefiniteLength();
        checkOidSize(definiteLength);
        if (definiteLength > derInputStream.available()) {
            throw new IOException("ObjectIdentifier length exceeds data available.  Length: " + definiteLength + ", Available: " + derInputStream.available());
        }
        derInputStream.getBytes(this.encoding = new byte[definiteLength]);
        check(this.encoding);
        return;
    }
    
    ObjectIdentifier(DerInputBuffer derInputBuffer) throws IOException {
        this.encoding = null;
        this.components = null;
        this.componentLen = -1;
        this.componentsCalculated = false;
        DerInputStream derInputStream = new DerInputStream(derInputBuffer);
        int available = derInputStream.available();
        checkOidSize(available);
        derInputStream.getBytes(this.encoding = new byte[available]);
        check(this.encoding);
    }
    
    private void init(int[] array, int n) throws IOException {
        int n2 = 0;
        byte[] array2 = new byte[n * 5 + 1];
        int n3;
        if (array[1] < Integer.MAX_VALUE - array[0] * 40) {
            n3 = n2 + pack7Oid(array[0] * 40 + array[1], array2, n2);
        }
        else {
            n3 = n2 + pack7Oid(BigInteger.valueOf((long)array[1]).add(BigInteger.valueOf((long)(array[0] * 40))), array2, n2);
        }
        for (int i = 2; i < n; ++i) {
            n3 += pack7Oid(array[i], array2, n3);
            checkOidSize(n3);
        }
        System.arraycopy(array2, 0, this.encoding = new byte[n3], 0, n3);
    }
    
    public static ObjectIdentifier newInternal(int[] array) {
        try {
            return new ObjectIdentifier(array);
        }
        catch (IOException cause) {
            throw new RuntimeException(cause);
        }
    }
    
    void encode(DerOutputStream derOutputStream) throws IOException {
        derOutputStream.write((byte)6, this.encoding);
    }
    
    @Deprecated
    public boolean equals(ObjectIdentifier objectIdentifier) {
        return this.equals(objectIdentifier);
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof ObjectIdentifier && Arrays.equals(this.encoding, ((ObjectIdentifier)o).encoding));
    }
    
    @Override
    public int hashCode() {
        return Arrays.hashCode(this.encoding);
    }
    
    private int[] toIntArray() {
        int length = this.encoding.length;
        int[] copy = new int[20];
        int newLength = 0;
        int i = 0;
        for (int j = 0; j < length; ++j) {
            if ((this.encoding[j] & 0x80) == 0x0) {
                if (j - i + 1 > 4) {
                    BigInteger o = new BigInteger(pack(this.encoding, i, j - i + 1, 7, 8));
                    if (i == 0) {
                        copy[newLength++] = 2;
                        BigInteger subtract = o.subtract(BigInteger.valueOf(80L));
                        if (subtract.compareTo(BigInteger.valueOf(2147483647L)) == 1) {
                            return null;
                        }
                        copy[newLength++] = subtract.intValue();
                    }
                    else {
                        if (o.compareTo(BigInteger.valueOf(2147483647L)) == 1) {
                            return null;
                        }
                        copy[newLength++] = o.intValue();
                    }
                }
                else {
                    int n = 0;
                    for (; i <= j; ++i) {
                        n = (n << 7 | (this.encoding[i] & 0x7F));
                    }
                    if (i == 0) {
                        if (n < 80) {
                            copy[newLength++] = n / 40;
                            copy[newLength++] = n % 40;
                        }
                        else {
                            copy[newLength++] = 2;
                            copy[newLength++] = n - 80;
                        }
                    }
                    else {
                        copy[newLength++] = n;
                    }
                }
                i = j + 1;
            }
            if (newLength >= copy.length) {
                copy = Arrays.copyOf(copy, newLength + 10);
            }
        }
        return Arrays.copyOf(copy, newLength);
    }
    
    @Override
    public String toString() {
        String stringForm = this.stringForm;
        if (stringForm == null) {
            int length = this.encoding.length;
            StringBuffer sb = new StringBuffer(length * 4);
            int i = 0;
            for (int j = 0; j < length; ++j) {
                if ((this.encoding[j] & 0x80) == 0x0) {
                    if (i != 0) {
                        sb.append('.');
                    }
                    if (j - i + 1 > 4) {
                        BigInteger obj = new BigInteger(pack(this.encoding, i, j - i + 1, 7, 8));
                        if (i == 0) {
                            sb.append("2.");
                            sb.append(obj.subtract(BigInteger.valueOf(80L)));
                        }
                        else {
                            sb.append(obj);
                        }
                    }
                    else {
                        int k = 0;
                        for (; i <= j; ++i) {
                            k = (k << 7 | (this.encoding[i] & 0x7F));
                        }
                        if (i == 0) {
                            if (k < 80) {
                                sb.append(k / 40);
                                sb.append('.');
                                sb.append(k % 40);
                            }
                            else {
                                sb.append("2.");
                                sb.append(k - 80);
                            }
                        }
                        else {
                            sb.append(k);
                        }
                    }
                    i = j + 1;
                }
            }
            stringForm = sb.toString();
            this.stringForm = stringForm;
            return stringForm;
        }
        return stringForm;
    }
    
    private static byte[] pack(byte[] array, int n, int n2, int n3, int n4) {
        assert n3 > 0 && n3 <= 8 : "input NUB must be between 1 and 8";
        assert n4 > 0 && n4 <= 8 : "output NUB must be between 1 and 8";
        if (n3 == n4) {
            return (byte[])array.clone();
        }
        int n5 = n2 * n3;
        byte[] array2 = new byte[(n5 + n4 - 1) / n4];
        int n7;
        int n8;
        for (int i = 0, n6 = (n5 + n4 - 1) / n4 * n4 - n5; i < n5; n8 = n6 / n4, array2[n8] |= (array[n + i / n3] + 256 >> n3 - i % n3 - n7 & (1 << n7) - 1) << n4 - n6 % n4 - n7, i += n7, n6 += n7) {
            n7 = n3 - i % n3;
            if (n7 > n4 - n6 % n4) {
                n7 = n4 - n6 % n4;
            }
        }
        return array2;
    }
    
    private static int pack7Oid(byte[] array, int n, int n2, byte[] array2, int n3) {
        byte[] pack = pack(array, n, n2, 8, 7);
        int n4 = pack.length - 1;
        int n5;
        for (int i = pack.length - 2; i >= 0; n5 = i, pack[n5] |= 0x80, --i) {
            if (pack[i] != 0) {
                n4 = i;
            }
        }
        System.arraycopy(pack, n4, array2, n3, pack.length - n4);
        return pack.length - n4;
    }
    
    private static int pack8(byte[] array, int n, int n2, byte[] array2, int n3) {
        byte[] pack = pack(array, n, n2, 7, 8);
        int n4 = pack.length - 1;
        for (int i = pack.length - 2; i >= 0; --i) {
            if (pack[i] != 0) {
                n4 = i;
            }
        }
        System.arraycopy(pack, n4, array2, n3, pack.length - n4);
        return pack.length - n4;
    }
    
    private static int pack7Oid(int n, byte[] array, int n2) {
        return pack7Oid(new byte[] { (byte)(n >> 24), (byte)(n >> 16), (byte)(n >> 8), (byte)n }, 0, 4, array, n2);
    }
    
    private static int pack7Oid(BigInteger bigInteger, byte[] array, int n) {
        byte[] byteArray = bigInteger.toByteArray();
        return pack7Oid(byteArray, 0, byteArray.length, array, n);
    }
    
    private static void check(byte[] array) throws IOException {
        int length = array.length;
        if (length < 1 || (array[length - 1] & 0x80) != 0x0) {
            throw new IOException("ObjectIdentifier() -- Invalid DER encoding, not ended");
        }
        int i = 0;
        while (i < length) {
            if (array[i] == -128 && (i == 0 || (array[i - 1] & 0x80) == 0x0)) {
                throw new IOException("ObjectIdentifier() -- Invalid DER encoding, useless extra octet detected");
            }
            ++i;
        }
        return;
    }
    
    private static void checkCount(int n) throws IOException {
        if (n < 2) {
            throw new IOException("ObjectIdentifier() -- Must be at least two oid components ");
        }
        return;
    }
    
    private static void checkFirstComponent(int n) throws IOException {
        if (n < 0 || n > 2) {
            throw new IOException("ObjectIdentifier() -- First oid component is invalid ");
        }
        return;
    }
    
    private static void checkFirstComponent(BigInteger bigInteger) throws IOException {
        if (bigInteger.signum() == -1 || bigInteger.compareTo(BigInteger.valueOf(2L)) == 1) {
            throw new IOException("ObjectIdentifier() -- First oid component is invalid ");
        }
        return;
    }
    
    private static void checkSecondComponent(int n, int n2) throws IOException {
        if (n2 < 0 || (n != 2 && n2 > 39)) {
            throw new IOException("ObjectIdentifier() -- Second oid component is invalid ");
        }
        return;
    }
    
    private static void checkSecondComponent(int n, BigInteger bigInteger) throws IOException {
        if (bigInteger.signum() == -1 || (n != 2 && bigInteger.compareTo(BigInteger.valueOf(39L)) == 1)) {
            throw new IOException("ObjectIdentifier() -- Second oid component is invalid ");
        }
        return;
    }
    
    private static void checkOtherComponent(int n, int n2) throws IOException {
        if (n2 < 0) {
            throw new IOException("ObjectIdentifier() -- oid component #" + (n + 1) + " must be non-negative ");
        }
        return;
    }
    
    private static void checkOtherComponent(int n, BigInteger bigInteger) throws IOException {
        if (bigInteger.signum() == -1) {
            throw new IOException("ObjectIdentifier() -- oid component #" + (n + 1) + " must be non-negative ");
        }
        return;
    }
    
    private static void checkOidSize(int i) throws IOException {
        if (i > 4096) {
            throw new IOException("ObjectIdentifier encoded length exceeds the restriction in JDK (OId length(>=): " + i + ", Restriction: " + 4096 + ")");
        }
        return;
    }
    
    static class HugeOidNotSupportedByOldJDK implements Serializable
    {
        private static final long serialVersionUID = 1L;
        static ObjectIdentifier.HugeOidNotSupportedByOldJDK theOne;
        
        static {
            ObjectIdentifier.HugeOidNotSupportedByOldJDK.theOne = new ObjectIdentifier.HugeOidNotSupportedByOldJDK();
        }
    }
}
